@@ -1,14 +1,15 @@
-{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE CPP #-}
 module Main where
 
 import Control.Monad (void)
+import Control.Monad.State (modify)
 import Data.Maybe (fromMaybe)
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
 #endif
 import qualified Graphics.Vty as V
 import Lens.Micro ((^.))
+import Lens.Micro.Mtl
 
 import qualified Brick.AttrMap as A
 import qualified Brick.Main as M
@@ -39,26 +40,28 @@ drawUI l = [ui]
                               , C.hCenter $ str "Press Esc to exit."
                               ]
 
-appEvent :: L.List () Char -> T.BrickEvent () e -> T.EventM () (T.Next (L.List () Char))
-appEvent l (T.VtyEvent e) =
+appEvent :: T.BrickEvent () e -> T.EventM () (L.List () Char) ()
+appEvent (T.VtyEvent e) =
     case e of
-        V.EvKey (V.KChar '+') [] ->
-            let el = nextElement (L.listElements l)
-                pos = Vec.length $ l^.(L.listElementsL)
-            in M.continue $ L.listInsert pos el l
+        V.EvKey (V.KChar '+') [] -> do
+            els <- use L.listElementsL
+            let el = nextElement els
+                pos = Vec.length els
+            modify $ L.listInsert pos el
 
-        V.EvKey (V.KChar '-') [] ->
-            case l^.(L.listSelectedL) of
-                Nothing -> M.continue l
-                Just i  -> M.continue $ L.listRemove i l
+        V.EvKey (V.KChar '-') [] -> do
+            sel <- use L.listSelectedL
+            case sel of
+                Nothing -> return ()
+                Just i  -> modify $ L.listRemove i
 
-        V.EvKey V.KEsc [] -> M.halt l
+        V.EvKey V.KEsc [] -> M.halt
 
-        ev -> M.continue =<< (L.handleListEventVi L.handleListEvent) ev l
+        ev -> L.handleListEventVi L.handleListEvent ev
     where
       nextElement :: Vec.Vector Char -> Char
       nextElement v = fromMaybe '?' $ Vec.find (flip Vec.notElem v) (Vec.fromList ['a' .. 'z'])
-appEvent l _ = M.continue l
+appEvent _ = return ()
 
 listDrawElement :: (Show a) => Bool -> a -> Widget ()
 listDrawElement sel a =
@@ -71,7 +74,7 @@ initialState :: L.List () Char
 initialState = L.list () (Vec.fromList ['a','b','c']) 1
 
 customAttr :: A.AttrName
-customAttr = L.listSelectedAttr <> "custom"
+customAttr = L.listSelectedAttr <> A.attrName "custom"
 
 theMap :: A.AttrMap
 theMap = A.attrMap V.defAttr
@@ -85,7 +88,7 @@ theApp =
     M.App { M.appDraw = drawUI
           , M.appChooseCursor = M.showFirstCursor
           , M.appHandleEvent = appEvent
-          , M.appStartEvent = return
+          , M.appStartEvent = return ()
           , M.appAttrMap = const theMap
           }
 
