@@ -60,7 +60,7 @@ hCenterWith mChar p =
            c <- getContext
            let rWidth = result^.imageL.to imageWidth
                rHeight = result^.imageL.to imageHeight
-               remainder = max 0 $ c^.availWidthL - (leftPaddingAmount * 2)
+               remainder = max 0 $ c^.availWidthL - (rWidth + (leftPaddingAmount * 2))
                leftPaddingAmount = max 0 $ (c^.availWidthL - rWidth) `div` 2
                rightPaddingAmount = max 0 $ leftPaddingAmount + remainder
                leftPadding = charFill (c^.attrL) ch leftPaddingAmount rHeight
@@ -153,7 +153,16 @@ centerAbout l p =
       c <- getContext
       let centerW = c^.availWidthL `div` 2
           centerH = c^.availHeightL `div` 2
-          off = Location ( centerW - l^.columnL
-                         , centerH - l^.rowL
+          off = Location ( centerW - l^.locationColumnL
+                         , centerH - l^.locationRowL
                          )
-      render $ translateBy off p
+      result <- render $ translateBy off p
+
+      -- Pad the result so it consumes available space
+      let rightPaddingAmt = max 0 $ c^.availWidthL - imageWidth (result^.imageL)
+          bottomPaddingAmt = max 0 $ c^.availHeightL - imageHeight (result^.imageL)
+          rightPadding = charFill (c^.attrL) ' ' rightPaddingAmt (imageHeight $ result^.imageL)
+          bottomPadding = charFill (c^.attrL) ' ' (imageWidth $ result^.imageL) bottomPaddingAmt
+          paddedImg = horizCat [vertCat [result^.imageL, bottomPadding], rightPadding]
+
+      return $ result & imageL .~ paddedImg
