@@ -101,7 +101,8 @@ module Brick.Widgets.Core
   , withHScrollBarHandles
   , withVScrollBarRenderer
   , withHScrollBarRenderer
-  , ScrollbarRenderer(..)
+  , VScrollbarRenderer(..)
+  , HScrollbarRenderer(..)
   , verticalScrollbarRenderer
   , horizontalScrollbarRenderer
   , scrollbarAttr
@@ -718,12 +719,11 @@ renderBox br ws =
                             (concatMap extents allTranslatedResults)
                             newBorders
 
-catDynBorder
-    :: Lens' (Edges BorderSegment) BorderSegment
-    -> Lens' (Edges BorderSegment) BorderSegment
-    -> DynBorder
-    -> DynBorder
-    -> Maybe DynBorder
+catDynBorder :: Lens' (Edges BorderSegment) BorderSegment
+             -> Lens' (Edges BorderSegment) BorderSegment
+             -> DynBorder
+             -> DynBorder
+             -> Maybe DynBorder
 catDynBorder towardsA towardsB a b
     -- Currently, we check if the 'BorderStyle's are exactly the same. In the
     -- future, it might be nice to relax this restriction. For example, if a
@@ -741,12 +741,11 @@ catDynBorder towardsA towardsB a b
     = Just (a & dbSegmentsL.towardsB.bsDrawL .~ True)
     | otherwise = Nothing
 
-catDynBorders
-    :: Lens' (Edges BorderSegment) BorderSegment
-    -> Lens' (Edges BorderSegment) BorderSegment
-    -> I.IMap DynBorder
-    -> I.IMap DynBorder
-    -> I.IMap DynBorder
+catDynBorders :: Lens' (Edges BorderSegment) BorderSegment
+              -> Lens' (Edges BorderSegment) BorderSegment
+              -> I.IMap DynBorder
+              -> I.IMap DynBorder
+              -> I.IMap DynBorder
 catDynBorders towardsA towardsB am bm = I.mapMaybe id
     $ I.intersectionWith (catDynBorder towardsA towardsB) am bm
 
@@ -756,9 +755,8 @@ catDynBorders towardsA towardsB am bm = I.mapMaybe id
 -- images to keep the image in sync with the border information.
 --
 -- The input borders are assumed to be disjoint. This property is not checked.
-catBorders
-    :: (border ~ BM.BorderMap DynBorder, rewrite ~ I.IMap V.Image)
-    => BoxRenderer n -> border -> border -> ((rewrite, rewrite), border)
+catBorders :: (border ~ BM.BorderMap DynBorder, rewrite ~ I.IMap V.Image)
+           => BoxRenderer n -> border -> border -> ((rewrite, rewrite), border)
 catBorders br r l = if lCoord + 1 == rCoord
     then ((lRe, rRe), lr')
     else ((I.empty, I.empty), lr)
@@ -787,20 +785,20 @@ catBorders br r l = if lCoord + 1 == rCoord
 -- overlap and are strictly increasing in the primary direction), produce: a
 -- list of rewrites for the lo and hi directions of each border, respectively,
 -- and the borders describing the fully concatenated object.
-catAllBorders ::
-    BoxRenderer n ->
-    [BM.BorderMap DynBorder] ->
-    ([(I.IMap V.Image, I.IMap V.Image)], BM.BorderMap DynBorder)
+catAllBorders :: BoxRenderer n
+              -> [BM.BorderMap DynBorder]
+              -> ([(I.IMap V.Image, I.IMap V.Image)], BM.BorderMap DynBorder)
 catAllBorders _ [] = ([], BM.empty)
 catAllBorders br (bm:bms) = (zip ([I.empty]++los) (his++[I.empty]), bm') where
     (rewrites, bm') = runState (traverse (state . catBorders br) bms) bm
     (his, los) = unzip rewrites
 
-rewriteEdge ::
-    (Int -> V.Image -> V.Image) ->
-    (Int -> V.Image -> V.Image) ->
-    ([V.Image] -> V.Image) ->
-    I.IMap V.Image -> V.Image -> V.Image
+rewriteEdge :: (Int -> V.Image -> V.Image)
+            -> (Int -> V.Image -> V.Image)
+            -> ([V.Image] -> V.Image)
+            -> I.IMap V.Image
+            -> V.Image
+            -> V.Image
 rewriteEdge splitLo splitHi combine = (combine .) . go . offsets 0 . I.unsafeToAscList where
 
     -- convert absolute positions into relative ones
@@ -1083,15 +1081,16 @@ translateBy off p =
 -- | Given a widget, translate it to position it relative to the
 -- upper-left coordinates of a reported extent with the specified
 -- positioning offset. If the specified name has no reported extent,
--- this just draws the specified widget with no special positioning.
+-- this draws nothing on the basis that it only makes sense to draw what
+-- was requested when the relative position can be known.
 --
 -- This is only useful for positioning something in a higher layer
 -- relative to a reported extent in a lower layer. Any other use is
--- likely to result in the specified widget being rendered as-is with
--- no translation. This is because this function relies on information
--- about lower layer renderings in order to work; using it with a
--- resource name that wasn't rendered in a lower layer will result in
--- this being equivalent to @id@.
+-- likely to result in the specified widget not being rendered. This
+-- is because this function relies on information about lower layer
+-- renderings in order to work; using it with a resource name that
+-- wasn't rendered in a lower layer will result in this being equivalent
+-- to @emptyWidget@.
 --
 -- For example, if you have two layers @topLayer@ and @bottomLayer@,
 -- then a widget drawn in @bottomLayer@ with @reportExtent Foo@ can be
@@ -1102,7 +1101,7 @@ relativeTo n off w =
     Widget (hSize w) (vSize w) $ do
         mExt <- lookupReportedExtent n
         case mExt of
-            Nothing -> render w
+            Nothing -> render emptyWidget
             Just ext -> render $ translateBy (extentUpperLeft ext <> off) w
 
 -- | Crop the specified widget on the left by the specified number of
@@ -1113,8 +1112,11 @@ cropLeftBy cols p =
       result <- render p
       let amt = V.imageWidth (result^.imageL) - cols
           cropped img = if amt < 0 then V.emptyImage else V.cropLeft amt img
-      return $ addResultOffset (Location (-1 * cols, 0))
-             $ result & imageL %~ cropped
+      render $ Widget (hSize p) (vSize p) $
+               withReaderT (availWidthL .~ amt) $
+                   cropResultToContext $
+                       addResultOffset (Location (-1 * cols, 0)) $
+                           result & imageL %~ cropped
 
 -- | Crop the specified widget to the specified size from the left.
 -- Defers to the cropped widget for growth policy.
@@ -1136,7 +1138,8 @@ cropRightBy cols p =
       result <- render p
       let amt = V.imageWidth (result^.imageL) - cols
           cropped img = if amt < 0 then V.emptyImage else V.cropRight amt img
-      return $ result & imageL %~ cropped
+      withReaderT (availWidthL .~ amt) $
+          cropResultToContext $ result & imageL %~ cropped
 
 -- | Crop the specified widget to the specified size from the right.
 -- Defers to the cropped widget for growth policy.
@@ -1158,8 +1161,11 @@ cropTopBy rows p =
       result <- render p
       let amt = V.imageHeight (result^.imageL) - rows
           cropped img = if amt < 0 then V.emptyImage else V.cropTop amt img
-      return $ addResultOffset (Location (0, -1 * rows))
-             $ result & imageL %~ cropped
+      render $ Widget (hSize p) (vSize p) $
+               withReaderT (availHeightL .~ amt) $
+                   cropResultToContext $
+                       addResultOffset (Location (0, -1 * rows)) $
+                           result & imageL %~ cropped
 
 -- | Crop the specified widget to the specified size from the top.
 -- Defers to the cropped widget for growth policy.
@@ -1181,7 +1187,8 @@ cropBottomBy rows p =
       result <- render p
       let amt = V.imageHeight (result^.imageL) - rows
           cropped img = if amt < 0 then V.emptyImage else V.cropBottom amt img
-      return $ result & imageL %~ cropped
+      withReaderT (availHeightL .~ amt) $
+          cropResultToContext $ result & imageL %~ cropped
 
 -- | Crop the specified widget to the specified size from the bottom.
 -- Defers to the cropped widget for growth policy.
@@ -1253,7 +1260,6 @@ cached n w =
             allClickables <- use clickableNamesL
             return [extentName e | e <- renderResult^.extentsL, extentName e `elem` allClickables]
 
-
 cacheLookup :: (Ord n) => n -> RenderM n (Maybe ([n], Result n))
 cacheLookup n = do
     cache <- lift $ gets (^.renderCacheL)
@@ -1285,20 +1291,21 @@ withVScrollBarHandles w =
 -- | Render vertical viewport scroll bars in the specified widget with
 -- the specified renderer. This is only needed if you want to override
 -- the use of the default renderer, 'verticalScrollbarRenderer'.
-withVScrollBarRenderer :: ScrollbarRenderer n -> Widget n -> Widget n
+withVScrollBarRenderer :: VScrollbarRenderer n -> Widget n -> Widget n
 withVScrollBarRenderer r w =
     Widget (hSize w) (vSize w) $
         withReaderT (ctxVScrollBarRendererL .~ Just r) (render w)
 
 -- | The default renderer for vertical viewport scroll bars. Override
 -- with 'withVScrollBarRenderer'.
-verticalScrollbarRenderer :: ScrollbarRenderer n
+verticalScrollbarRenderer :: VScrollbarRenderer n
 verticalScrollbarRenderer =
-    ScrollbarRenderer { renderScrollbar = fill '█'
-                      , renderScrollbarTrough = fill ' '
-                      , renderScrollbarHandleBefore = str "^"
-                      , renderScrollbarHandleAfter = str "v"
-                      }
+    VScrollbarRenderer { renderVScrollbar = fill '█'
+                       , renderVScrollbarTrough = fill ' '
+                       , renderVScrollbarHandleBefore = str "^"
+                       , renderVScrollbarHandleAfter = str "v"
+                       , scrollbarWidthAllocation = 1
+                       }
 
 -- | Enable horizontal scroll bars on all viewports in the specified
 -- widget and draw them with the specified orientation.
@@ -1343,20 +1350,21 @@ withHScrollBarHandles w =
 -- | Render horizontal viewport scroll bars in the specified widget with
 -- the specified renderer. This is only needed if you want to override
 -- the use of the default renderer, 'horizontalScrollbarRenderer'.
-withHScrollBarRenderer :: ScrollbarRenderer n -> Widget n -> Widget n
+withHScrollBarRenderer :: HScrollbarRenderer n -> Widget n -> Widget n
 withHScrollBarRenderer r w =
     Widget (hSize w) (vSize w) $
         withReaderT (ctxHScrollBarRendererL .~ Just r) (render w)
 
 -- | The default renderer for horizontal viewport scroll bars. Override
 -- with 'withHScrollBarRenderer'.
-horizontalScrollbarRenderer :: ScrollbarRenderer n
+horizontalScrollbarRenderer :: HScrollbarRenderer n
 horizontalScrollbarRenderer =
-    ScrollbarRenderer { renderScrollbar = fill '█'
-                      , renderScrollbarTrough = fill ' '
-                      , renderScrollbarHandleBefore = str "<"
-                      , renderScrollbarHandleAfter = str ">"
-                      }
+    HScrollbarRenderer { renderHScrollbar = fill '█'
+                       , renderHScrollbarTrough = fill ' '
+                       , renderHScrollbarHandleBefore = str "<"
+                       , renderHScrollbarHandleAfter = str ">"
+                       , scrollbarHeightAllocation = 1
+                       }
 
 -- | Render the specified widget in a named viewport with the
 -- specified type. This permits widgets to be scrolled without being
@@ -1373,11 +1381,11 @@ horizontalScrollbarRenderer =
 -- don't like the appearance of the resulting scroll bars (defaults:
 -- 'verticalScrollbarRenderer' and 'horizontalScrollbarRenderer'),
 -- you can customize how they are drawn by making your own
--- 'ScrollbarRenderer' and using 'withVScrollBarRenderer' and/or
--- 'withHScrollBarRenderer'. Note that when you enable scrollbars, the
--- content of your viewport will lose one column of available space if
--- vertical scroll bars are enabled and one row of available space if
--- horizontal scroll bars are enabled.
+-- 'VScrollbarRenderer' or 'HScrollbarRenderer' and using
+-- 'withVScrollBarRenderer' and/or 'withHScrollBarRenderer'. Note that
+-- when you enable scrollbars, the content of your viewport will lose
+-- one column of available space if vertical scroll bars are enabled and
+-- one row of available space if horizontal scroll bars are enabled.
 --
 -- If a viewport receives more than one visibility request, then the
 -- visibility requests are merged with the inner visibility request
@@ -1441,8 +1449,8 @@ viewport vpname typ p =
           newSize = (newWidth, newHeight)
           newWidth = c^.availWidthL - vSBWidth
           newHeight = c^.availHeightL - hSBHeight
-          vSBWidth = maybe 0 (const 1) vsOrientation
-          hSBHeight = maybe 0 (const 1) hsOrientation
+          vSBWidth = maybe 0 (const $ scrollbarWidthAllocation vsRenderer) vsOrientation
+          hSBHeight = maybe 0 (const $ scrollbarHeightAllocation hsRenderer) hsOrientation
           doInsert (Just vp) = Just $ vp & vpSize .~ newSize
           doInsert Nothing = Just newVp
 
@@ -1542,7 +1550,8 @@ viewport vpname typ p =
       let addVScrollbar = case vsOrientation of
               Nothing -> id
               Just orientation ->
-                  let sb = verticalScrollbar vsRenderer vpname
+                  let sb = verticalScrollbar vsRenderer orientation
+                                                        vpname
                                                         vsbClickableConstr
                                                         showVHandles
                                                         (vpFinal^.vpSize._2)
@@ -1555,7 +1564,8 @@ viewport vpname typ p =
           addHScrollbar = case hsOrientation of
               Nothing -> id
               Just orientation ->
-                  let sb = horizontalScrollbar hsRenderer vpname
+                  let sb = horizontalScrollbar hsRenderer orientation
+                                                          vpname
                                                           hsbClickableConstr
                                                           showHHandles
                                                           (vpFinal^.vpSize._1)
@@ -1609,7 +1619,7 @@ maybeClick :: (Ord n)
 maybeClick _ Nothing _ w = w
 maybeClick n (Just f) el w = clickable (f el n) w
 
--- | Build a vertical scroll bar using the specified render and
+-- | Build a vertical scroll bar using the specified renderer and
 -- settings.
 --
 -- You probably don't want to use this directly; instead,
@@ -1618,8 +1628,13 @@ maybeClick n (Just f) el w = clickable (f el n) w
 -- render a scroll bar of your own, you can do so outside the @viewport@
 -- context.
 verticalScrollbar :: (Ord n)
-                  => ScrollbarRenderer n
+                  => VScrollbarRenderer n
                   -- ^ The renderer to use.
+                  -> VScrollBarOrientation
+                  -- ^ The scroll bar orientation. The orientation
+                  -- governs how additional padding is added to
+                  -- the scroll bar if it is smaller than it space
+                  -- allocation according to 'scrollbarWidthAllocation'.
                   -> n
                   -- ^ The viewport name associated with this scroll
                   -- bar.
@@ -1634,24 +1649,35 @@ verticalScrollbar :: (Ord n)
                   -> Int
                   -- ^ The total viewport content height.
                   -> Widget n
-verticalScrollbar vsRenderer n constr False vpHeight vOffset contentHeight =
-    verticalScrollbar' vsRenderer n constr vpHeight vOffset contentHeight
-verticalScrollbar vsRenderer n constr True vpHeight vOffset contentHeight =
-    vBox [ maybeClick n constr SBHandleBefore $
-           hLimit 1 $ withDefAttr scrollbarHandleAttr $ renderScrollbarHandleBefore vsRenderer
-         , verticalScrollbar' vsRenderer n constr vpHeight vOffset contentHeight
-         , maybeClick n constr SBHandleAfter $
-           hLimit 1 $ withDefAttr scrollbarHandleAttr $ renderScrollbarHandleAfter vsRenderer
-         ]
+verticalScrollbar vsRenderer o n constr showHandles vpHeight vOffset contentHeight =
+    hLimit (scrollbarWidthAllocation vsRenderer) $
+    applyPadding $
+    if showHandles
+       then vBox [ vLimit 1 $
+                   maybeClick n constr SBHandleBefore $
+                   withDefAttr scrollbarHandleAttr $ renderVScrollbarHandleBefore vsRenderer
+                 , sbBody
+                 , vLimit 1 $
+                   maybeClick n constr SBHandleAfter $
+                   withDefAttr scrollbarHandleAttr $ renderVScrollbarHandleAfter vsRenderer
+                 ]
+       else sbBody
+    where
+        sbBody = verticalScrollbar' vsRenderer n constr vpHeight vOffset contentHeight
+        applyPadding = case o of
+            OnLeft -> padRight Max
+            OnRight -> padLeft Max
 
 verticalScrollbar' :: (Ord n)
-                   => ScrollbarRenderer n
+                   => VScrollbarRenderer n
                    -- ^ The renderer to use.
                    -> n
                    -- ^ The viewport name associated with this scroll
                    -- bar.
                    -> Maybe (ClickableScrollbarElement -> n -> n)
-                   -- ^ Constructor for clickable scroll bar element names.
+                   -- ^ Constructor for clickable scroll bar element
+                   -- names. Will be given the element name and the
+                   -- viewport name.
                    -> Int
                    -- ^ The total viewport height in effect.
                    -> Int
@@ -1660,7 +1686,7 @@ verticalScrollbar' :: (Ord n)
                    -- ^ The total viewport content height.
                    -> Widget n
 verticalScrollbar' vsRenderer _ _ vpHeight _ 0 =
-    hLimit 1 $ vLimit vpHeight $ renderScrollbarTrough vsRenderer
+    vLimit vpHeight $ renderVScrollbarTrough vsRenderer
 verticalScrollbar' vsRenderer n constr vpHeight vOffset contentHeight =
     Widget Fixed Greedy $ do
         c <- getContext
@@ -1692,22 +1718,21 @@ verticalScrollbar' vsRenderer n constr vpHeight vOffset contentHeight =
 
             sbAbove = maybeClick n constr SBTroughBefore $
                       withDefAttr scrollbarTroughAttr $ vLimit sbOffset $
-                      renderScrollbarTrough vsRenderer
+                      renderVScrollbarTrough vsRenderer
             sbBelow = maybeClick n constr SBTroughAfter $
                       withDefAttr scrollbarTroughAttr $ vLimit (ctxHeight - (sbOffset + sbSize)) $
-                      renderScrollbarTrough vsRenderer
+                      renderVScrollbarTrough vsRenderer
             sbMiddle = maybeClick n constr SBBar $
-                       withDefAttr scrollbarAttr $ vLimit sbSize $ renderScrollbar vsRenderer
+                       withDefAttr scrollbarAttr $ vLimit sbSize $ renderVScrollbar vsRenderer
 
-            sb = hLimit 1 $
-                 if sbSize == ctxHeight
+            sb = if sbSize == ctxHeight
                  then vLimit sbSize $
-                      renderScrollbarTrough vsRenderer
+                      renderVScrollbarTrough vsRenderer
                  else vBox [sbAbove, sbMiddle, sbBelow]
 
         render sb
 
--- | Build a horizontal scroll bar using the specified render and
+-- | Build a horizontal scroll bar using the specified renderer and
 -- settings.
 --
 -- You probably don't want to use this directly; instead, use
@@ -1716,14 +1741,21 @@ verticalScrollbar' vsRenderer n constr vpHeight vOffset contentHeight =
 -- render a scroll bar of your own, you can do so outside the @viewport@
 -- context.
 horizontalScrollbar :: (Ord n)
-                    => ScrollbarRenderer n
+                    => HScrollbarRenderer n
                     -- ^ The renderer to use.
+                    -> HScrollBarOrientation
+                    -- ^ The scroll bar orientation. The orientation
+                    -- governs how additional padding is added
+                    -- to the scroll bar if it is smaller
+                    -- than it space allocation according to
+                    -- 'scrollbarHeightAllocation'.
                     -> n
                     -- ^ The viewport name associated with this scroll
                     -- bar.
                     -> Maybe (ClickableScrollbarElement -> n -> n)
                     -- ^ Constructor for clickable scroll bar element
-                    -- names.
+                    -- names. Will be given the element name and the
+                    -- viewport name.
                     -> Bool
                     -- ^ Whether to show handles.
                     -> Int
@@ -1733,18 +1765,27 @@ horizontalScrollbar :: (Ord n)
                     -> Int
                     -- ^ The total viewport content width.
                     -> Widget n
-horizontalScrollbar hsRenderer n constr False vpWidth hOffset contentWidth =
-    horizontalScrollbar' hsRenderer n constr vpWidth hOffset contentWidth
-horizontalScrollbar hsRenderer n constr True vpWidth hOffset contentWidth =
-    hBox [ maybeClick n constr SBHandleBefore $
-           vLimit 1 $ withDefAttr scrollbarHandleAttr $ renderScrollbarHandleBefore hsRenderer
-         , horizontalScrollbar' hsRenderer n constr vpWidth hOffset contentWidth
-         , maybeClick n constr SBHandleAfter $
-           vLimit 1 $ withDefAttr scrollbarHandleAttr $ renderScrollbarHandleAfter hsRenderer
-         ]
+horizontalScrollbar hsRenderer o n constr showHandles vpWidth hOffset contentWidth =
+    vLimit (scrollbarHeightAllocation hsRenderer) $
+    applyPadding $
+    if showHandles
+       then hBox [ hLimit 1 $
+                   maybeClick n constr SBHandleBefore $
+                   withDefAttr scrollbarHandleAttr $ renderHScrollbarHandleBefore hsRenderer
+                 , sbBody
+                 , hLimit 1 $
+                   maybeClick n constr SBHandleAfter $
+                   withDefAttr scrollbarHandleAttr $ renderHScrollbarHandleAfter hsRenderer
+                 ]
+       else sbBody
+    where
+        sbBody = horizontalScrollbar' hsRenderer n constr vpWidth hOffset contentWidth
+        applyPadding = case o of
+            OnTop -> padBottom Max
+            OnBottom -> padTop Max
 
 horizontalScrollbar' :: (Ord n)
-                     => ScrollbarRenderer n
+                     => HScrollbarRenderer n
                      -- ^ The renderer to use.
                      -> n
                      -- ^ The viewport name associated with this scroll
@@ -1760,7 +1801,7 @@ horizontalScrollbar' :: (Ord n)
                      -- ^ The total viewport content width.
                      -> Widget n
 horizontalScrollbar' hsRenderer _ _ vpWidth _ 0 =
-    vLimit 1 $ hLimit vpWidth $ renderScrollbarTrough hsRenderer
+    hLimit vpWidth $ renderHScrollbarTrough hsRenderer
 horizontalScrollbar' hsRenderer n constr vpWidth hOffset contentWidth =
     Widget Greedy Fixed $ do
         c <- getContext
@@ -1791,17 +1832,16 @@ horizontalScrollbar' hsRenderer n constr vpWidth hOffset contentWidth =
 
             sbLeft = maybeClick n constr SBTroughBefore $
                      withDefAttr scrollbarTroughAttr $ hLimit sbOffset $
-                     renderScrollbarTrough hsRenderer
+                     renderHScrollbarTrough hsRenderer
             sbRight = maybeClick n constr SBTroughAfter $
                       withDefAttr scrollbarTroughAttr $ hLimit (ctxWidth - (sbOffset + sbSize)) $
-                      renderScrollbarTrough hsRenderer
+                      renderHScrollbarTrough hsRenderer
             sbMiddle = maybeClick n constr SBBar $
-                       withDefAttr scrollbarAttr $ hLimit sbSize $ renderScrollbar hsRenderer
+                       withDefAttr scrollbarAttr $ hLimit sbSize $ renderHScrollbar hsRenderer
 
-            sb = vLimit 1 $
-                 if sbSize == ctxWidth
+            sb = if sbSize == ctxWidth
                  then hLimit sbSize $
-                      renderScrollbarTrough hsRenderer
+                      renderHScrollbarTrough hsRenderer
                  else hBox [sbLeft, sbMiddle, sbRight]
 
         render sb
